package xtext.workshop.advanced.ui.quickfix;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.xtext.ui.editor.model.IXtextDocument;
import org.eclipse.xtext.ui.editor.model.edit.IModification;
import org.eclipse.xtext.ui.editor.model.edit.IModificationContext;
import org.eclipse.xtext.ui.editor.model.edit.ISemanticModification;
import org.eclipse.xtext.ui.editor.quickfix.DefaultQuickfixProvider;
import org.eclipse.xtext.ui.editor.quickfix.Fix;
import org.eclipse.xtext.ui.editor.quickfix.IssueResolutionAcceptor;
import org.eclipse.xtext.util.Strings;
import org.eclipse.xtext.validation.Issue;

import xtext.workshop.advanced.touristguideDsl.City;
import xtext.workshop.advanced.touristguideDsl.Sight;
import xtext.workshop.advanced.touristguideDsl.TouristguideDslFactory;
import xtext.workshop.advanced.validation.TouristguideDslJavaValidator;

public class TouristguideDslQuickfixProvider extends DefaultQuickfixProvider {

	@Fix(TouristguideDslJavaValidator.INVALID_NAME)
	public void capitalizeName(final Issue issue,
			IssueResolutionAcceptor acceptor) {
		acceptor.accept(issue, "Capitalize name", "Capitalize the name \""
				+ issue.getData()[0] + "\".", "upcase.png", new IModification() {
			public void apply(IModificationContext context)	throws BadLocationException {
				IXtextDocument xtextDocument = context.getXtextDocument();
				String firstLetter = xtextDocument.get(issue.getOffset() + 1, 1);
				xtextDocument.replace(issue.getOffset() + 1, 1,
						firstLetter.toUpperCase());
			}
		});
	}

	@Fix(TouristguideDslJavaValidator.CITY_NOT_INTERESTING)
	public void addSightToCity(final Issue issue,
			IssueResolutionAcceptor acceptor) {

		acceptor.accept(issue, "Add sight to make city more interesting",
				"Add a random sight, to make the city look more interesting.",
				null, new ISemanticModification() {
					public void apply(EObject element,
							IModificationContext context) throws Exception {
						City c = (City) element;

						Sight sight = TouristguideDslFactory.eINSTANCE.createSight();
						sight.setName("Central Station");
						sight.setDescription("The famous central station of "
								+ Strings.toFirstUpper(c.getName()) + ".");
						c.getSights().add(sight);
					}
				});
	}
}